/* Copyright (C) 2022 GSI Helmholtzzentrum fuer Schwerionenforschung, Darmstadt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Sergei Zharko [committer], Maksym Zyzak */

/// \file    CaTrackFinderWindow.h
/// \authors S.Zharko <s.zharko@gsi.de> (interface), M.Zyzak (original algorithm)
/// \brief   A class wrapper over clones merger algorithm for the CA track finder (declaration)
/// \since   22.07.2022

#pragma once  // include this header only once per compilation unit

#include "CaBranch.h"
#include "CaHit.h"
#include "CaSimd.h"
#include "CaTrackParam.h"
#include "CaVector.h"


namespace cbm::algo::ca
{

  class Track;
  class Triplet;
  class Framework;

  namespace
  {
    using namespace cbm::algo;  // to keep ca:: prefices in the code
  }

  /// Class implements a clones merger algorithm for the CA track finder
  ///
  class TrackFinderWindow {
  public:
    /// Default constructor
    TrackFinderWindow(ca::Framework& algo);

    /// Destructor
    ~TrackFinderWindow();

    /// Copy constructor
    TrackFinderWindow(const TrackFinderWindow&) = delete;

    /// Move constructor
    TrackFinderWindow(TrackFinderWindow&&) = delete;

    /// Copy assignment operator
    TrackFinderWindow& operator=(const TrackFinderWindow&) = delete;

    /// Move assignment operator
    TrackFinderWindow& operator=(TrackFinderWindow&&) = delete;

    void ReadWindowData();
    void CaTrackFinderSlice();

    void CAFindTrack(int ista, ca::Branch& best_tr, const ca::Triplet* curr_trip, ca::Branch& curr_tr,
                     unsigned char min_best_l, ca::Branch* new_tr);

  private:
    ///-------------------------------
    /// Private methods

    bool checkTripletMatch(const ca::Triplet& l, const ca::Triplet& r, fscal& dchi2) const;


  private:
    ///-------------------------------
    /// Data members

    ca::Framework& frAlgo;  ///< Reference to the main track finder algorithm class
  };

}  // namespace cbm::algo::ca
