/* Copyright (C) 2023 FIAS Frankfurt Institute for Advanced Studies, Frankfurt / Main
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Felix Weiglhofer [committer] */
#ifndef CBM_ALGO_GLOBAL_RECO_H
#define CBM_ALGO_GLOBAL_RECO_H

#include "tof/CalibratorChain.h"
#include "tof/HitfinderChain.h"

#include <xpu/host.h>

#include "EventbuildChain.h"
#include "HistogramSender.h"
#include "SubChain.h"
#include "UnpackChain.h"
#include "ca/TrackingChain.h"
#include "global/RecoResults.h"
#include "sts/HitfinderChain.h"

namespace fles
{
  class Timeslice;
}  // namespace fles

namespace cbm::algo
{
  class Options;

  struct ProcessingMonitor {
    xpu::timings fTime;  //< total processing time
  };

  class Reco : SubChain {
  public:
    Reco();
    ~Reco();

    Reco(const Reco&) = delete;
    Reco& operator=(const Reco&) = delete;
    Reco(Reco&&)                 = delete;
    Reco& operator=(Reco&&) = delete;

    void Init(const Options&);
    RecoResults Run(const fles::Timeslice&);
    void Finalize();
    void PrintTimings(xpu::timings&);

  private:
    bool fInitialized = false;
    ChainContext fContext;
    xpu::timings fTimesliceTimesAcc;
    std::shared_ptr<HistogramSender> fSender;

    size_t prevTsId = 0;

    // STS
    UnpackChain fUnpack;
    sts::HitfinderChain fStsHitFinder;

    // TOF
    tof::HitfinderChain fTofHitFinder;
    tof::CalibratorChain fTofCalibrator;

    // Eventbuilding
    std::unique_ptr<evbuild::EventbuildChain> fEventBuild;

    // Tracking
    std::unique_ptr<TrackingChain> fTracking;

    void Validate(const Options& opts);

    void QueueUnpackerMetrics(const fles::Timeslice&, const UnpackMonitorData&, const DigiData&);
    void QueueStsRecoMetrics(const sts::HitfinderMonitor&);
    void QueueTofRecoMetrics(const tof::HitfindMonitorData&);
    void QueueTofCalibMetrics(const tof::CalibrateMonitorData&);
    void QueueEvbuildMetrics(const evbuild::EventbuildChainMonitorData&);
    void QueueTrackingMetrics(const ca::TrackingMonitorData&);
    void QueueProcessingMetrics(const ProcessingMonitor&);
  };
}  // namespace cbm::algo

#endif
